SystemOrganization addCategory: #'Nile-Base-Classes'!
SystemOrganization addCategory: #'Nile-Base-Errors'!
SystemOrganization addCategory: #'Nile-Base-Traits'!

Trait named: #NSTByteReading
	uses: {}
	category: 'Nile-Base-Traits'!

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 5/31/2008 13:39'!
boolean
	"Answer the next boolean value from this (binary) stream."

	^ self next ~= 0
! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 8/1/2007 19:12'!
int16
	"Answer the next signed, 16-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + (self next).
	n >= 16r8000 ifTrue: [n := n - 16r10000].
	^ n
! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 8/1/2007 19:12'!
int32
	"Answer the next signed, 32-bit integer from this (binary) stream."
	"Details: As a fast check for negative number, check the high bit of the first digit"

	| n firstDigit |
	n := firstDigit := self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	firstDigit >= 128 ifTrue: [n := -16r100000000 + n].  "decode negative 32-bit integer"
	^ n
! !

!NSTByteReading methodsFor: 'testing' stamp: 'dc 7/27/2007 09:35'!
isBinary
	^ true! !

!NSTByteReading methodsFor: 'reading' stamp: 'dc 5/15/2007 16:12'!
next
	self explicitRequirement ! !

!NSTByteReading methodsFor: 'reading' stamp: 'dc 5/15/2007 16:12'!
next: aNumber
	self explicitRequirement! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 5/19/2007 12:06'!
nextBoolean
	"Answer the next boolean value from this (binary) stream."

	^ self next ~= 0
! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 5/15/2007 16:12'!
nextInt32
	"Read a 32-bit signed integer from the next 4 bytes"
	| s |
	s := 0.
	1 to: 4 do: [:i | s := (s bitShift: 8) + self next].
	(s bitAnd: 16r80000000) = 0
		ifTrue: [^ s]
		ifFalse: [^ -1 - s bitInvert32]! !

!NSTByteReading methodsFor: 'reading' stamp: 'dc 5/15/2007 16:12'!
nextInto: aCollection
	self explicitRequirement! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 5/15/2007 16:12'!
nextLittleEndianNumber: n 
	"Answer the next n bytes as a positive Integer or LargePositiveInteger, where the bytes are ordered from least significant to most significant."

	| bytes s |
	bytes := self next: n.
	s := 0.
	n to: 1 by: -1 do: [:i | s := (s bitShift: 8) bitOr: (bytes at: i)].
	^ s! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 5/15/2007 16:12'!
nextNumber: n 
	"Answer the next n bytes as a positive Integer or LargePositiveInteger."
	| s |
	s := 0.
	1 to: n do: 
		[:i | s := (s bitShift: 8) bitOr: self next asInteger].
	^ s normalize! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 5/15/2007 16:12'!
nextString
	"Read a string from the receiver. The first byte is the length of the string, unless it is greater than 192, in which case the first four bytes encode the length."

	| length aByteArray |
	length := self next.		"first byte."
	length >= 192 ifTrue: [length := length - 192.
		1 to: 3 do: [:ii | length := length * 256 + self next]].
	aByteArray := ByteArray new: length.

	self nextInto: aByteArray.
	^aByteArray asString.
! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 5/21/2007 10:25'!
nextWord
	"Answer the next two bytes from the receiver as an Integer."

	| high low |
	high := self next.
	low := self next.
	^(high asInteger bitShift: 8) + low asInteger! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 8/1/2007 18:51'!
string
	"Answer the next string from this (binary) stream."

	| size |
	size := self uint16.
	^ (self next: size) asString
! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 8/1/2007 18:50'!
uint16
	"Answer the next unsigned, 16-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + (self next).
	^ n
! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 8/1/2007 18:59'!
uint24
	"Answer the next unsigned, 24-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	^ n
! !

!NSTByteReading methodsFor: 'binary reading' stamp: 'dc 8/1/2007 19:00'!
uint32
	"Answer the next unsigned, 32-bit integer from this (binary) stream."

	| n |
	n := self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	n := (n bitShift: 8) + self next.
	^ n
! !

Trait named: #NSTByteWriting
	uses: {}
	category: 'Nile-Base-Traits'!

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 8/1/2007 19:09'!
boolean: aBoolean
	"Store the given boolean value on this (binary) stream."

	self nextPut: (aBoolean ifTrue: [1] ifFalse: [0]).
! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 8/1/2007 19:10'!
int16: anInteger
	"Store the given signed, 16-bit integer on this (binary) stream."

	| n |
	(anInteger < -16r8000) | (anInteger >= 16r8000)
		ifTrue: [self error: 'outside 16-bit integer range'].

	anInteger < 0
		ifTrue: [n := 16r10000 + anInteger]
		ifFalse: [n := anInteger].
	self nextPut: (n digitAt: 2).
	self nextPut: (n digitAt: 1).
! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 8/1/2007 19:11'!
int32: anInteger
	"Store the given signed, 32-bit integer on this (binary) stream."

	| n |
	(anInteger < -16r80000000) | (anInteger >= 16r80000000)
		ifTrue: [self error: 'outside 32-bit integer range'].

	anInteger < 0
		ifTrue: [n := 16r100000000 + anInteger]
		ifFalse: [n := anInteger].
	self nextPut: (n digitAt: 4).
	self nextPut: (n digitAt: 3).
	self nextPut: (n digitAt: 2).
	self nextPut: (n digitAt: 1).
! !

!NSTByteWriting methodsFor: 'testing' stamp: 'dc 7/27/2007 09:35'!
isBinary
	^ true! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 5/19/2007 12:33'!
nextInt32Put: int32
	"Write a signed integer to the next 4 bytes"
	| pos |
	pos := int32 < 0
		ifTrue: [(0-int32) bitInvert32 + 1]
		ifFalse: [int32].
	1 to: 4 do: [:i | self nextPut: (pos digitAt: 5-i)].
	^ int32! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 5/19/2007 12:44'!
nextLittleEndianNumber: n put: value
	"Writes value in the following n bytes of the stream where the bytes are ordered from least significant to most significant."
	| bytes |
	bytes := ByteArray new: n.
	1 to: n do: [: i | bytes at: i put: (value digitAt: i)].
	self nextPutAll: bytes! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 5/19/2007 12:48'!
nextNumber: n put: v 
	"Append to the receiver the argument, v, which is a positive 
	SmallInteger or a LargePositiveInteger, as the next n bytes.
	Possibly pad with leading zeros."

	1 to: n do: [:i | self nextPut: (v digitAt: n+1-i)].
	^ v
! !

!NSTByteWriting methodsFor: 'writing' stamp: 'dc 5/19/2007 12:41'!
nextPut: anObject
	self explicitRequirement ! !

!NSTByteWriting methodsFor: 'writing' stamp: 'dc 5/19/2007 12:41'!
nextPutAll: aCollection
	self explicitRequirement ! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 5/19/2007 12:50'!
nextStringPut: s 
	"Append the string, s, to the receiver.  Only used by DataStream.  Max size of 64*256*256*256."

	| length |
	(length := s size) < 192
		ifTrue: [self nextPut: length]
		ifFalse: 
			[self nextPut: (length digitAt: 4)+192.
			self nextPut: (length digitAt: 3).
			self nextPut: (length digitAt: 2).
			self nextPut: (length digitAt: 1)].
	self nextPutAll: s asByteArray.
	^s! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 5/19/2007 12:53'!
nextWordPut: aWord 
	"Append to the receiver an Integer as the next two bytes."

	self nextPut: ((aWord bitShift: -8) bitAnd: 255).
	self nextPut: (aWord bitAnd: 255).
	^aWord! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 8/1/2007 19:12'!
string: aString
	"Store the given string on this (binary) stream. The string must contain 65535 or fewer characters."

	aString size > 16rFFFF ifTrue: [self error: 'string too long for this format'].
	self uint16: aString size.
	self nextPutAll: aString asByteArray.
! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 8/1/2007 19:07'!
uint16: anInteger
	"Store the given unsigned, 16-bit integer on this (binary) stream."

	(anInteger < 0) | (anInteger >= 16r10000)
		ifTrue: [self error: 'outside unsigned 16-bit integer range'].

	self nextPut: (anInteger digitAt: 2).
	self nextPut: (anInteger digitAt: 1).
! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 8/1/2007 19:07'!
uint24: anInteger
	"Store the given unsigned, 24-bit integer on this (binary) stream."

	(anInteger < 0) | (anInteger >= 16r1000000)
		ifTrue: [self error: 'outside unsigned 24-bit integer range'].

	self nextPut: (anInteger digitAt: 3).
	self nextPut: (anInteger digitAt: 2).
	self nextPut: (anInteger digitAt: 1).
! !

!NSTByteWriting methodsFor: 'binary writing' stamp: 'dc 8/1/2007 19:07'!
uint32: anInteger
	"Store the given unsigned, 32-bit integer on this (binary) stream."

	(anInteger < 0) | (anInteger >= 16r100000000)
		ifTrue: [self error: 'outside unsigned 32-bit integer range'].

	self nextPut: (anInteger digitAt: 4).
	self nextPut: (anInteger digitAt: 3).
	self nextPut: (anInteger digitAt: 2).
	self nextPut: (anInteger digitAt: 1).
! !

Trait named: #NSTCharacterReading
	uses: {}
	category: 'Nile-Base-Traits'!

!NSTCharacterReading methodsFor: 'reading strings' stamp: 'dc 5/5/2007 13:57'!
nextLine
	| line |
	line := self upToElementSatisfying: [:element | (element = Character cr) | (element = Character lf)].

	"In case encoding is CRLF, takes the lf too:"
	self peekFor: Character lf.
	
	^ line! !

!NSTCharacterReading methodsFor: 'reading strings' stamp: 'dc 7/26/2007 16:15'!
skipSeparators
	[self atEnd not and: [self peek isSeparator]]
		whileTrue: [self next]! !

Trait named: #NSTCharacterWriting
	uses: {}
	category: 'Nile-Base-Traits'!

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 3/30/2007 17:25'!
cr
	self nextPut: Character cr! !

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 5/4/2007 18:39'!
cr: anInteger
	anInteger timesRepeat: [self cr]! !

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 5/14/2007 16:24'!
crtab
	self cr; tab.! !

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 5/14/2007 16:24'!
crtab: amount
	self
		cr;
		tab: amount.! !

!NSTCharacterWriting methodsFor: 'accessing' stamp: 'dc 3/30/2007 17:24'!
nextPut: anObject
	^ self explicitRequirement
	! !

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 3/30/2007 17:24'!
space
	self nextPut: Character space! !

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 5/4/2007 18:39'!
space: anInteger
	anInteger timesRepeat: [self space]! !

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 3/30/2007 17:25'!
tab
	self nextPut: Character tab! !

!NSTCharacterWriting methodsFor: 'spaces' stamp: 'dc 5/4/2007 18:39'!
tab: anInteger
	anInteger timesRepeat: [self tab]! !

Trait named: #NSTStream
	uses: {}
	category: 'Nile-Base-Traits'!

Trait named: #NSTGettableStream
	uses: NSTStream
	category: 'Nile-Base-Traits'!

NSTGettableStream classTrait
	uses: NSTStream classTrait!

Trait named: #NSTDecoder
	uses: NSTGettableStream
	category: 'Nile-Base-Traits'!

NSTDecoder classTrait
	uses: NSTGettableStream classTrait!

!NSTDecoder methodsFor: 'private - accessing' stamp: 'dc 5/3/2007 13:08'!
atEnd: aBoolean
	self explicitRequirement! !

!NSTDecoder methodsFor: 'private - accessing' stamp: 'dc 5/5/2007 15:42'!
effectiveNext
	"Reads the input stream and returns a computed value based on it. For private use only. Please read the comment of NSTDecoder."
	"This method has to use #inputStream to access the original stream. This method MUST throw NSStreamAtEndError if it can't read anything interesting in the input stream."
	self explicitRequirement! !

!NSTDecoder methodsFor: 'initialize-release' stamp: 'dc 5/9/2007 18:26'!
initializeDecoder
	"Initialize the decoder. See comment in NSTDecoder. The class using this trait must call this method before being useable."
	self atEnd: false.
	self nextValue: ([self effectiveNext]
						on: NSStreamAtEndError 
						do: [:e | self atEnd: true]).! !

!NSTDecoder methodsFor: 'accessing' stamp: 'dc 5/5/2007 15:28'!
next
	| currentValue |
	currentValue := self nextValue.
	self nextValue: ([self effectiveNext]
						on: NSStreamAtEndError
						do: [:e | self atEnd: true]).
	^ currentValue! !

!NSTDecoder methodsFor: 'private - accessing' stamp: 'dc 5/3/2007 13:08'!
nextValue
	self explicitRequirement! !

!NSTDecoder methodsFor: 'private - accessing' stamp: 'dc 5/3/2007 13:08'!
nextValue: anObject
	self explicitRequirement! !

!NSTDecoder methodsFor: 'accessing' stamp: 'dc 5/3/2007 13:09'!
peek
	^ self atEnd
		ifFalse: [self nextValue]
		ifTrue: [nil].! !

!NSTGettableStream methodsFor: 'testing' stamp: 'dc 2/28/2007 15:52'!
atEnd
	"Returns a boolean indicating whether the receiver is at the end of its values (no more entry to read)."
	^ self explicitRequirement! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 7/26/2007 16:34'!
basicNext
	^ self next! !

!NSTGettableStream methodsFor: 'enumerating' stamp: 'dc 4/4/2007 13:38'!
do: aBlock
	"Reads elements and passes them one after the other as aBlock argument. Take care that your stream is not an infinite stream while using this method."
	[self atEnd]
		whileFalse: [aBlock value: self next]! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 5/5/2007 15:58'!
next
	"Reads the next object in the stream and returns it. Please ensure that the stream is not at its end before calling #next. Behavior is not defined if the stream is at its end."
	self explicitRequirement! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 5/5/2007 15:59'!
next: amount
	"Reads the next amount objects in the stream and returns a collection containing them in the same order."
	| moves elements |
	moves := amount.
	elements := OrderedCollection new: amount.
	[self atEnd not and: [moves > 0]]
		whileTrue: [elements add: self next.
					moves := moves - 1].
	^ elements as: self outputCollectionClass! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 5/18/2007 14:25'!
nextInto: aCollection
	"Read the next elements of the receiver into aCollection.
	Return aCollection or a partial copy if less than aCollection
	size elements have been read."
	1 to: aCollection size do:
		[:i | self atEnd ifTrue: [^ aCollection copyFrom: 1 to: i-1].
			aCollection at: i put: self next].
	^ aCollection! !

!NSTGettableStream methodsFor: 'testing' stamp: 'dc 3/1/2007 15:32'!
nextMatchFor: anObject
	"Reads the next object from the stream and returns true if the object is equal to the argument and false otherwise."
	^ self next = anObject! !

!NSTGettableStream methodsFor: 'private - accessing' stamp: 'dc 5/5/2007 16:05'!
outputCollectionClass
	"This method must return a Collection class. This class will be used to return collection of elements when #next: or #upTo: are sent. In fact, any kind of class which answers to #newFrom: can be used as soon as #newFrom: accepts a collection as a parameter."
	self explicitRequirement! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 3/1/2007 12:46'!
peek
	"Returns the next object in the receiver's future sequence values without advancing the receiver's position. Returns nil if the receiver is at end of stream."
	self explicitRequirement! !

!NSTGettableStream methodsFor: 'testing' stamp: 'dc 4/23/2007 14:06'!
peekFor: anObject
	"Answer false and do not move over the next element if it is not equal to 
	the argument, anObject, or if the receiver is at the end. Answer true 
	and increment the position, if the next element is equal to anObject."
	
	"This definition does not conform to ANSI standard. However most smalltalks use this definition which is more useful. ANSI definition of #peekFor: is just an equality test between the object peeked up and the parameter."
	
	^ (self atEnd not and: [self peek = anObject])
		ifTrue: [self next. true]
		ifFalse: [false]! !

!NSTGettableStream methodsFor: 'positioning' stamp: 'dc 3/1/2007 14:42'!
skip: amount
	"Skips the next amount objects in the receiver's future sequence values. Just go to the end if there is not enough element in the stream."
	amount
		timesRepeat: [self atEnd ifTrue: [^self].
			 		 self next.].! !

!NSTGettableStream methodsFor: 'positioning' stamp: 'dc 5/5/2007 11:37'!
skipTo: anObject
	"Sets the stream to read the object just after the next occurence of the argument and returns true. If the argument is not found before the end of the stream is encountered, false is returned."
	self do: [:each | each = anObject ifTrue: [^ true]].
	^ false! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 5/5/2007 12:14'!
upTo: anObject
	"Returns a collection of all of the objects in the receiver up to, but not including, the next occurrence ofthe argument. Sets the stream to read the object just after the next occurrence of the argument. If the argument is not found and the end of the stream is encountered, an ordered collection of the objects read is returned."
	^ self upToElementSatisfying: [:each | each = anObject]! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 5/18/2007 14:34'!
upToElementSatisfying: oneArgumentBlock
	"Returns a collection of all of the objects in the receiver up to, but not including, the next element satisfying oneArgumentBlock. Sets the stream to read the object just after the element satisfying oneArgumentBlock. If no element satisfies the block and the end of the stream is encountered, an ordered collection of the objects read is returned."
	| collection |
	collection := OrderedCollection new.
	self do: [:element|
				(oneArgumentBlock value: element)
					ifTrue: [^ collection as: self outputCollectionClass].
				collection add: element].
	^ collection as: self outputCollectionClass ! !

!NSTGettableStream methodsFor: 'reading' stamp: 'dc 5/5/2007 12:20'!
upToEnd
	"Answers remaining elements. Position the stream to its end."
	^ self upToElementSatisfying: [:each | false]! !

Trait named: #NSTPositionableStream
	uses: NSTStream
	category: 'Nile-Base-Traits'!

NSTPositionableStream classTrait
	uses: NSTStream classTrait!

Trait named: #NSTGettablePositionableStream
	uses: NSTGettableStream + NSTPositionableStream @ {#basicBack->#back}
	category: 'Nile-Base-Traits'!

NSTGettablePositionableStream classTrait
	uses: NSTGettableStream classTrait + NSTPositionableStream classTrait!

!NSTGettablePositionableStream methodsFor: 'positioning' stamp: 'dc 9/20/2007 16:34'!
back
	"Goes back one position in the stream and returns the new next element."
	"The behavior of #back is different from the one of the original Squeak. This is because Squeak considers there is a current element, a previous element and a following element. I believe it is a mistake. Streams only have future and past elements. There is no notion of current."
	self basicBack.
	^ self peek! !

!NSTGettablePositionableStream methodsFor: 'positioning' stamp: 'dc 9/20/2007 16:40'!
backUpTo: subCollection 
	| pattern startMatch |
	pattern := subCollection reversed readStream.
	startMatch := nil.
	[pattern atEnd] whileFalse: 
			[self position = 0 ifTrue: [^ false].
			self skip: -1.
			self next = pattern next 
				ifTrue: [pattern position = 1 ifTrue: [startMatch := self position]]
				ifFalse: 
					[pattern position: 0.
					startMatch ifNotNil: 
							[self position: startMatch - 1.
							startMatch := nil]].
			self skip: -1].
	self position: startMatch.
	^ true! !

!NSTGettablePositionableStream methodsFor: 'positioning' stamp: 'dc 5/19/2007 11:32'!
basicBack
	"Goes back one position in the stream."
	self atStart
		ifTrue: [NSStreamAtStartError signal: 'No way to go back because stream is at begining.'].
		
	self position: self position - 1.! !

!NSTGettablePositionableStream methodsFor: 'positioning' stamp: 'dc 9/20/2007 16:40'!
match: subCollection
	"Set the access position of the receiver to be past the next occurrence of the subCollection. Answer whether subCollection is found.  No wildcards and case does matter."

	| pattern startMatch |
	pattern := subCollection nsReadStream.
	startMatch := nil.
	[pattern atEnd] whileFalse: 
		[self atEnd ifTrue: [^ false].
		(self next) = (pattern peek) 
			ifTrue: [pattern atStart
						ifTrue: [startMatch := self position].
					pattern next]
			ifFalse: [startMatch
						ifNotNil: [self position: startMatch.
		  						  startMatch := nil]]].
	^ true! !

!NSTGettablePositionableStream methodsFor: 'reading' stamp: 'dc 9/20/2007 16:47'!
nextDelimited: terminator
	"Answer the contents of the receiver, up to the next terminator character. Doubled terminators indicate an embedded terminator character.  For example: 'this '' was a quote'. Start postioned before the initial terminator."

	| out ch |
	out := self outputCollectionClass new nsWriteStream.
	self atEnd ifTrue: [^ ''].
	self next. "Jump after the first terminator."
	[(ch := self next) == nil] whileFalse: [
		(ch == terminator) ifTrue: [
			self peek == terminator ifTrue: [
				self next.  "skip doubled terminator"
			] ifFalse: [
				^ out contents  "terminator is not doubled; we're done!!"
			].
		].
		out nextPut: ch.
	].
	^ out contents! !

!NSTGettablePositionableStream methodsFor: 'positioning' stamp: 'dc 9/20/2007 16:33'!
skip: amount
	"Moves relatively in the stream. Go forward amount elements. Go backward if amount is negative."
	"Overrides TGettableStream>>skip: for efficiency and backward possibility."
	self position: ((self position + amount) min: self size max: 0)! !

!NSTPositionableStream methodsFor: 'testing' stamp: 'damiencassou 2/13/2009 16:46'!
atEnd
	"Answers true if and only if there is no more element to read."
	^ self position = self size! !

!NSTPositionableStream methodsFor: 'testing' stamp: 'dc 5/5/2007 16:09'!
atStart
	"Answers true if and only if there is no element to read before the current position."
	^ self position = 0! !

!NSTPositionableStream methodsFor: 'positioning' stamp: 'dc 5/19/2007 11:32'!
back
	"Goes back one position in the stream."
	self atStart
		ifTrue: [NSStreamAtStartError signal: 'No way to go back because stream is at begining.'].
		
	self position: self position - 1.! !

!NSTPositionableStream methodsFor: 'testing' stamp: 'dc 5/5/2007 16:10'!
isEmpty
	"Returns a boolean indicating wheter there are any sequence values in the receiver. That is to say, there is no element before and no element after the current position."
	^ self size isZero! !

!NSTPositionableStream methodsFor: 'testing' stamp: 'dc 5/26/2007 12:00'!
isInBounds: position 
	^ position positive and: [position <= self size]! !

!NSTPositionableStream methodsFor: 'positioning' stamp: 'dc 5/5/2007 16:12'!
position
	"Returns the number of element before the current position in the stream."
	self explicitRequirement! !

!NSTPositionableStream methodsFor: 'positioning' stamp: 'dc 5/26/2007 12:01'!
position: newPosition
	"Sets the number of element before the position to be the parameter newPosition. 0 for the start of the stream. Throws an error if the parameter is lesser than 0 or greater than the size."
	(self isInBounds: newPosition)
		ifFalse: [NSInvalidArgumentError signal].
	self setPosition: newPosition.
	! !

!NSTPositionableStream methodsFor: 'positioning' stamp: 'dc 5/5/2007 16:17'!
reset
	"Resets the position of the receiver to be at the start of the stream."
	self position: 0.! !

!NSTPositionableStream methodsFor: 'private - accessing' stamp: 'dc 5/5/2007 16:18'!
setPosition: anInteger
	"Required accessor to a variable. No need to check anything here because the check is done in #position:."
	self explicitRequirement! !

!NSTPositionableStream methodsFor: 'positioning' stamp: 'dc 5/5/2007 16:19'!
setToEnd
	"Set the position of the stream to its end. No more element after the current position will stay after this call."
	self position: self size! !

!NSTPositionableStream methodsFor: 'accessing' stamp: 'dc 5/5/2007 16:24'!
size
	"Returns an integer indicating the number of elements we can position into."
	self explicitRequirement! !

Trait named: #NSTPuttableStream
	uses: NSTStream
	category: 'Nile-Base-Traits'!

NSTPuttableStream classTrait
	uses: NSTStream classTrait!

Trait named: #NSTPuttablePositionableStream
	uses: NSTPuttableStream + NSTPositionableStream
	category: 'Nile-Base-Traits'!

NSTPuttablePositionableStream classTrait
	uses: NSTPuttableStream classTrait + NSTPositionableStream classTrait!

!NSTPuttablePositionableStream methodsFor: 'writing' stamp: 'dc 3/13/2008 16:00'!
writeBack: anObject
	self atStart ifTrue: [NSStreamAtStartError signal: 'You can''t write back because you are at the start of your stream.'].
	self back.
	self nextPut: anObject.
	^ anObject! !

!NSTPuttableStream methodsFor: 'writing' stamp: 'dc 7/26/2007 16:35'!
basicNextPut: anObject
	^ self nextPut: anObject ! !

!NSTPuttableStream methodsFor: 'writing' stamp: 'dc 7/26/2007 16:35'!
basicNextPutAll: aCollection 
	^ self nextPutAll: aCollection.
! !

!NSTPuttableStream methodsFor: 'actions' stamp: 'dc 3/9/2007 09:48'!
flush
	"Ensure the backing store is updated. Default implementation does nothing."! !

!NSTPuttableStream methodsFor: 'writing' stamp: 'dc 4/4/2007 12:50'!
next: anInteger put: anObject
	"Stores anObject as the next anInteger number of elements. Answer anObject."
	anInteger timesRepeat: [self nextPut: anObject].
	^ anObject! !

!NSTPuttableStream methodsFor: 'writing' stamp: 'dc 4/4/2007 13:11'!
nextPut: anObject
	"Appends anObject to the receiver's past sequence values. If the receiver's future sequence values is not empty, removes its first object. Returns anObject."
	self explicitRequirement! !

!NSTPuttableStream methodsFor: 'writing' stamp: 'dc 4/4/2007 13:11'!
nextPutAll: aCollection
	"Enumerates the argument, adding each element to the receiver. Returns aCollection."
	aCollection do: [:each | self nextPut: each].
	^ aCollection! !

!NSTPuttableStream methodsFor: 'writing' stamp: 'dc 5/19/2007 12:03'!
print: anObject
	"Asks the object to print itself on the receiver."
	anObject printOn: self! !

!NSTStream methodsFor: 'actions' stamp: 'dc 7/25/2007 17:18'!
binary
! !

!NSTStream methodsFor: 'actions' stamp: 'dc 7/26/2007 16:27'!
close
	"Disassociate a stream from its backing store. Does nothing by default. Streams over files for example will have to close the file in this method."! !

!NSTStream methodsFor: 'testing' stamp: 'dc 8/2/2007 17:17'!
closed
	self deprecated: 'Please use #isClosed instead.'.
	self isClosed! !

!NSTStream methodsFor: 'testing' stamp: 'dc 7/27/2007 09:34'!
isBinary
	^ false! !

!NSTStream methodsFor: 'testing' stamp: 'dc 8/2/2007 15:30'!
isClosed
	^ false! !

!NSTStream methodsFor: 'testing' stamp: 'dc 7/27/2007 13:28'!
isStream
	^ true! !

Trait named: #NSTUpdateAdler32
	uses: {}
	category: 'Nile-Base-Traits'!

!NSTUpdateAdler32 methodsFor: 'crc' stamp: 'cyrille.delaunay 6/3/2009 10:12'!
updateAdler32: adler from: start to: stop in: aCollection
	"Update crc using the Adler32 checksum technique from RFC1950"
"
        unsigned long s1 = adler & 0xffff;
        unsigned long s2 = (adler >> 16) & 0xffff;
        int n;

        for (n = 0; n < len; n++) {
          s1 = (s1 + buf[n]) % BASE;
          s2 = (s2 + s1)     % BASE;
        }
        return (s2 << 16) + s1;
"
	| s1 s2 |
	s1 := adler bitAnd: 16rFFFF.
	s2 := (adler bitShift: -16) bitAnd: 16rFFFF.
	start to: stop do: [ :n | | b |
		b := aCollection byteAt: n.
		s1 := (s1 + b) \\ 65521.
		s2 := (s2 + s1) \\ 65521. ].
	^(s2 bitShift: 16) + s1! !

!NSTUpdateAdler32 methodsFor: 'crc' stamp: 'cyrille.delaunay 6/3/2009 10:16'!
updateCrc: oldCrc from: start to: stop in: aCollection
	"Answer an updated CRC for the range of bytes in aCollection"
	^self updateAdler32: oldCrc from: start to: stop in: aCollection.! !

Trait named: #NSTUpdateCrc
	uses: {}
	category: 'Nile-Base-Traits'!

!NSTUpdateCrc methodsFor: 'requirements' stamp: 'cyrille.delaunay 6/3/2009 10:20'!
crcTable

^#(16r00000000 16r77073096 16rEE0E612C 16r990951BA 16r076DC419
  16r706AF48F 16rE963A535 16r9E6495A3 16r0EDB8832 16r79DCB8A4
  16rE0D5E91E 16r97D2D988 16r09B64C2B 16r7EB17CBD 16rE7B82D07
  16r90BF1D91 16r1DB71064 16r6AB020F2 16rF3B97148 16r84BE41DE
  16r1ADAD47D 16r6DDDE4EB 16rF4D4B551 16r83D385C7 16r136C9856
  16r646BA8C0 16rFD62F97A 16r8A65C9EC 16r14015C4F 16r63066CD9
  16rFA0F3D63 16r8D080DF5 16r3B6E20C8 16r4C69105E 16rD56041E4
  16rA2677172 16r3C03E4D1 16r4B04D447 16rD20D85FD 16rA50AB56B
  16r35B5A8FA 16r42B2986C 16rDBBBC9D6 16rACBCF940 16r32D86CE3
  16r45DF5C75 16rDCD60DCF 16rABD13D59 16r26D930AC 16r51DE003A
  16rC8D75180 16rBFD06116 16r21B4F4B5 16r56B3C423 16rCFBA9599
  16rB8BDA50F 16r2802B89E 16r5F058808 16rC60CD9B2 16rB10BE924
  16r2F6F7C87 16r58684C11 16rC1611DAB 16rB6662D3D 16r76DC4190
  16r01DB7106 16r98D220BC 16rEFD5102A 16r71B18589 16r06B6B51F
  16r9FBFE4A5 16rE8B8D433 16r7807C9A2 16r0F00F934 16r9609A88E
  16rE10E9818 16r7F6A0DBB 16r086D3D2D 16r91646C97 16rE6635C01
  16r6B6B51F4 16r1C6C6162 16r856530D8 16rF262004E 16r6C0695ED
  16r1B01A57B 16r8208F4C1 16rF50FC457 16r65B0D9C6 16r12B7E950
  16r8BBEB8EA 16rFCB9887C 16r62DD1DDF 16r15DA2D49 16r8CD37CF3
  16rFBD44C65 16r4DB26158 16r3AB551CE 16rA3BC0074 16rD4BB30E2
  16r4ADFA541 16r3DD895D7 16rA4D1C46D 16rD3D6F4FB 16r4369E96A
  16r346ED9FC 16rAD678846 16rDA60B8D0 16r44042D73 16r33031DE5
  16rAA0A4C5F 16rDD0D7CC9 16r5005713C 16r270241AA 16rBE0B1010
  16rC90C2086 16r5768B525 16r206F85B3 16rB966D409 16rCE61E49F
  16r5EDEF90E 16r29D9C998 16rB0D09822 16rC7D7A8B4 16r59B33D17
  16r2EB40D81 16rB7BD5C3B 16rC0BA6CAD 16rEDB88320 16r9ABFB3B6
  16r03B6E20C 16r74B1D29A 16rEAD54739 16r9DD277AF 16r04DB2615
  16r73DC1683 16rE3630B12 16r94643B84 16r0D6D6A3E 16r7A6A5AA8
  16rE40ECF0B 16r9309FF9D 16r0A00AE27 16r7D079EB1 16rF00F9344
  16r8708A3D2 16r1E01F268 16r6906C2FE 16rF762575D 16r806567CB
  16r196C3671 16r6E6B06E7 16rFED41B76 16r89D32BE0 16r10DA7A5A
  16r67DD4ACC 16rF9B9DF6F 16r8EBEEFF9 16r17B7BE43 16r60B08ED5
  16rD6D6A3E8 16rA1D1937E 16r38D8C2C4 16r4FDFF252 16rD1BB67F1
  16rA6BC5767 16r3FB506DD 16r48B2364B 16rD80D2BDA 16rAF0A1B4C
  16r36034AF6 16r41047A60 16rDF60EFC3 16rA867DF55 16r316E8EEF
  16r4669BE79 16rCB61B38C 16rBC66831A 16r256FD2A0 16r5268E236
  16rCC0C7795 16rBB0B4703 16r220216B9 16r5505262F 16rC5BA3BBE
  16rB2BD0B28 16r2BB45A92 16r5CB36A04 16rC2D7FFA7 16rB5D0CF31
  16r2CD99E8B 16r5BDEAE1D 16r9B64C2B0 16rEC63F226 16r756AA39C
  16r026D930A 16r9C0906A9 16rEB0E363F 16r72076785 16r05005713
  16r95BF4A82 16rE2B87A14 16r7BB12BAE 16r0CB61B38 16r92D28E9B
  16rE5D5BE0D 16r7CDCEFB7 16r0BDBDF21 16r86D3D2D4 16rF1D4E242
  16r68DDB3F8 16r1FDA836E 16r81BE16CD 16rF6B9265B 16r6FB077E1
  16r18B74777 16r88085AE6 16rFF0F6A70 16r66063BCA 16r11010B5C
  16r8F659EFF 16rF862AE69 16r616BFFD3 16r166CCF45 16rA00AE278
  16rD70DD2EE 16r4E048354 16r3903B3C2 16rA7672661 16rD06016F7
  16r4969474D 16r3E6E77DB 16rAED16A4A 16rD9D65ADC 16r40DF0B66
  16r37D83BF0 16rA9BCAE53 16rDEBB9EC5 16r47B2CF7F 16r30B5FFE9
  16rBDBDF21C 16rCABAC28A 16r53B39330 16r24B4A3A6 16rBAD03605
  16rCDD70693 16r54DE5729 16r23D967BF 16rB3667A2E 16rC4614AB8
  16r5D681B02 16r2A6F2B94 16rB40BBE37 16rC30C8EA1 16r5A05DF1B
  16r2D02EF8D
).! !

!NSTUpdateCrc methodsFor: 'crc' stamp: 'cyrille.delaunay 6/3/2009 10:10'!
updateCrc: oldCrc from: start to: stop in: aCollection
	| newCrc |
	<primitive: 'primitiveUpdateGZipCrc32' module: 'ZipPlugin'>
	newCrc := oldCrc.
	start to: stop do:[:i|
		newCrc := (self crcTable at: ((newCrc bitXor: (aCollection byteAt: i)) 
				bitAnd: 255) + 1) bitXor: (newCrc bitShift: -8).
	].
	^newCrc! !

!String class methodsFor: '*nile-base' stamp: 'dc 9/27/2007 16:30'!
nsReadStreamClass
	^ NSStringStream! !

!String class methodsFor: '*nile-base' stamp: 'dc 9/27/2007 14:56'!
nsWriteStreamClass
	^ NSStringStream! !

!ByteArray class methodsFor: '*nile-base' stamp: 'dc 9/27/2007 16:31'!
nsReadStreamClass
	^ NSByteStream! !

!ByteArray class methodsFor: '*nile-base' stamp: 'dc 9/27/2007 14:58'!
nsWriteStreamClass
	^ NSByteStream! !

!OrderedCollection methodsFor: '*nile-base' stamp: 'dc 5/26/2007 22:10'!
makeRoomFor: anInteger
	| newArray |

	(self capacity - firstIndex - 1 - self size) >= anInteger
		ifTrue: [^self].

	newArray := Array new: self size + (self growSizeGuaranteeing: anInteger).
	newArray replaceFrom: 1 to: array size with: array startingAt: 1.
	array := newArray.
	^ self! !

!OrderedCollection methodsFor: '*nile-base' stamp: 'dc 5/26/2007 21:52'!
replaceFrom: start to: stop with: replacement startingAt: repStart
	"An ordered collection is growable, so a call to this can make the collection grow."
	self makeRoomFor: (stop - self capacity).
	array
		replaceFrom: start + firstIndex - 1
		to: stop + firstIndex - 1
		with: replacement 
		startingAt: repStart.
	lastIndex := lastIndex max: stop + firstIndex - 1.! !

SequenceableCollection subclass: #NSCollectionLimiter
	instanceVariableNames: 'lowerBound upperBound collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Classes'!

!NSCollectionLimiter classSide methodsFor: 'instance creation' stamp: 'dc 5/26/2007 20:11'!
on: aCollection from: lowerBound to: upperBound 
	^ (self new)
		setCollection: aCollection
			lowerBound: lowerBound
			upperBound: upperBound;
		yourself! !

!NSCollectionLimiter methodsFor: 'accessing' stamp: 'damiencassou 1/8/2009 15:51'!
at: index 
	^ collection at: (self indexFor: index)! !

!NSCollectionLimiter methodsFor: 'accessing' stamp: 'damiencassou 2/6/2009 16:55'!
at: index put: anObject
	^ collection at: (self indexFor: index) put: anObject! !

!NSCollectionLimiter methodsFor: 'private' stamp: 'dc 5/26/2007 20:17'!
indexFor: index 
	^ index + lowerBound - 1! !

!NSCollectionLimiter methodsFor: 'initialize-release' stamp: 'dc 5/26/2007 20:11'!
setCollection: aCollection lowerBound: anInteger upperBound: anotherInteger 
	collection := aCollection.
	lowerBound := anInteger.
	upperBound := anotherInteger! !

!NSCollectionLimiter methodsFor: 'accessing' stamp: 'DamienCassou 12/14/2009 09:53'!
size
	^ upperBound - lowerBound + 1! !

!SequenceableCollection class methodsFor: '*nile-Base' stamp: 'dc 9/27/2007 16:29'!
nsReadStreamClass
	^ NSCollectionStream ! !

!SequenceableCollection class methodsFor: '*nile-Base' stamp: 'dc 9/27/2007 15:44'!
nsStreamContents: aBlock
	| stream |
	stream := self nsWriteStreamClass ignoringElementsOf: (self new: 100).
	aBlock value: stream.
	^ stream contents! !

!SequenceableCollection class methodsFor: '*nile-Base' stamp: 'dc 9/27/2007 14:54'!
nsWriteStreamClass
	^ NSCollectionStream! !

!SequenceableCollection methodsFor: '*nile-base' stamp: 'dc 5/24/2007 09:51'!
growSizeGuaranteeing: anInteger 
	^ (self size // 4 max: 20) + anInteger! !

!SequenceableCollection methodsFor: '*nile-base' stamp: 'damiencassou 2/17/2009 17:48'!
insertInto: aSequenceableCollection from: anInteger to: anotherInteger
	^ aSequenceableCollection replaceFrom: anInteger to: anotherInteger with: self startingAt: 1! !

!SequenceableCollection methodsFor: '*nile-base' stamp: 'dc 5/24/2007 09:51'!
makeRoomFor: anInteger 
	| newCollection |
	newCollection := self species new: self size + (self growSizeGuaranteeing: anInteger).
	newCollection 
		replaceFrom: 1
		to: self size
		with: self
		startingAt: 1.
	^ newCollection! !

!SequenceableCollection methodsFor: '*nile-base' stamp: 'dc 5/14/2007 16:19'!
nsReadStream
	^ self class nsReadStreamClass on: self! !

!SequenceableCollection methodsFor: '*nile-base' stamp: 'dc 5/19/2007 12:28'!
nsWriteStream
	^ self class nsWriteStreamClass on: self! !

!SequenceableCollection methodsFor: '*nile-base' stamp: 'damiencassou 2/17/2009 17:55'!
writeAllElementsOf: aCollection startingAt: position
	"Asserts receiver is big enough"
	^ aCollection
		insertInto: self
		from: position
		to: aCollection size + position - 1! !

Error subclass: #NSError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Errors'!

NSError subclass: #NSInvalidArgumentError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Errors'!

NSError subclass: #NSNoMoreElementError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Errors'!

NSError subclass: #NSPositionError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Errors'!

NSPositionError subclass: #NSStreamAtEndError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Errors'!

NSPositionError subclass: #NSStreamAtStartError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Errors'!

NSError subclass: #NSStreamCreationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Errors'!

!Collection methodsFor: '*nile-base' stamp: 'damiencassou 2/17/2009 17:20'!
insertInto: aSequenceableCollection from: anInteger to: anotherInteger 
	^ aSequenceableCollection replaceFrom: anInteger to: anotherInteger with: self asArray startingAt: 1! !

Object subclass: #NSCollectionStream
	uses: NSTGettablePositionableStream + NSTPuttablePositionableStream - {#back}
	instanceVariableNames: 'collection position streamSize capacity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Classes'!

!NSCollectionStream commentStamp: 'dc 10/6/2007 15:29' prior: 0!
Basically, I'm iterating over collections. I can help you read and write on collections.

Instance variables:

- collection: the collection on which I'm iterating. The collection can change if it's becoming too small to put all the data.
- position: integer which reminds the current position of the stream in the collection. It *must* be interpreted as being between 2 elements. 0 corresponds to the begining of the stream, before all elements. 1 corresponds to the position between the first and the second element and so on.
- streamSize: integer to know where the real information stops in the collection. The value is always between 0 and the size of the collection. For example: the streamSize of "NSCollectionStream ignoringElementsOf: (String new: 10)" is 0 whereas the collection size is 10.!

NSCollectionStream class
	uses: NSTGettablePositionableStream classTrait + NSTPuttablePositionableStream classTrait
	instanceVariableNames: ''!

NSCollectionStream subclass: #NSByteStream
	uses: NSTByteReading - {#next. #nextInto:. #next:} + NSTByteWriting - {#nextPut:. #nextPutAll:}
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Classes'!

NSByteStream class
	uses: NSTByteReading classTrait + NSTByteWriting classTrait
	instanceVariableNames: ''!

!NSByteStream methodsFor: 'testing' stamp: 'dc 9/27/2007 15:31'!
isBinary
	^ true! !

!NSCollectionStream classSide methodsFor: 'instance creation' stamp: 'dc 3/10/2008 16:25'!
ignoringElementsOf: aCollection
	"Use this constructor if the elements in aCollection have NO meaning for your stream. The stream only uses the size of aCollection. The stream is positioned at its begining. You may want to use ACollectionClass streamContents: [:stream| ...] instead."
	^ super basicNew
		setCollection: aCollection;
		streamSize: 0;
		position: 0;
		initialize;
		yourself! !

!NSCollectionStream classSide methodsFor: 'instance creation' stamp: 'dc 10/6/2007 11:48'!
new
	^ NSStreamCreationError signal: 'Can''t use #new to create a new stream'! !

!NSCollectionStream classSide methodsFor: 'instance creation' stamp: 'dc 3/10/2008 16:23'!
on: aCollection
	"Use this constructor if the elements in aCollection have a meaning for your stream, i.e., if you want to be able to read them. The stream is positioned at its begining."
	^ super basicNew
		setCollection: aCollection;
		setPosition: 0;
		streamSize: aCollection size;
		initialize;
		yourself! !

!NSCollectionStream classSide methodsFor: 'instance creation' stamp: 'dc 10/7/2007 10:32'!
on: aCollection from: anInteger to: anotherInteger
	^ self on: (NSCollectionLimiter on: aCollection from: anInteger to: anotherInteger)! !

!NSCollectionStream classSide methodsFor: 'instance creation' stamp: 'dc 3/10/2008 16:23'!
with: aCollection
	"Use this constructor if the elements in aCollection have a meaning for your stream, i.e., if you want to be able to read them. The stream is positioned at its end."
	^ super basicNew
		setCollection: aCollection;
		streamSize: aCollection size;
		setToEnd;
		initialize;
		yourself! !

!NSCollectionStream methodsFor: 'accessing' stamp: 'dc 9/20/2007 16:28'!
collection
	^ collection! !

!NSCollectionStream methodsFor: 'accessing' stamp: 'dc 3/10/2008 15:37'!
contents
	"Return a collection containing the complete contents of the stream."
	^ collection 
		copyFrom: 1
		to: streamSize! !

!NSCollectionStream methodsFor: 'private' stamp: 'dc 9/20/2007 16:56'!
freeSpace
	"Returns the number of elements that can be written to the stream before the end of the collection"
	^ collection size - position! !

!NSCollectionStream methodsFor: 'private' stamp: 'dc 9/20/2007 16:55'!
hasRoomFor: anInteger
	"Verifies there is enough free space in the collection to write anInteger elements at the current position."
	^ self freeSpace >= anInteger! !

!NSCollectionStream methodsFor: 'private' stamp: 'dc 9/20/2007 22:01'!
makeRoomFor: anInteger
	"Make the collection bigger to be able to write anInteger elements after the current position."
	collection := collection makeRoomFor: anInteger! !

!NSCollectionStream methodsFor: 'reading' stamp: 'dc 3/10/2008 17:32'!
next
	"Reads the next object in the stream and returns it."
	<primitive: 65>
	^ self atEnd 
		ifFalse: [collection at:  (position :=  position + 1)]! !

!NSCollectionStream methodsFor: 'reading' stamp: 'dc 3/10/2008 15:51'!
next: amount 
	| oldPosition realAmount |
	position := (streamSize min: (oldPosition := position) + amount).
	realAmount := position - oldPosition.
	^ (self outputCollectionClass new: realAmount) 
		replaceFrom: 1
		to: realAmount
		with: collection
		startingAt: oldPosition + 1! !

!NSCollectionStream methodsFor: 'writing' stamp: 'dc 9/20/2007 16:54'!
nextPut: anObject
	"Appends anObject to the receiver's past sequence values. If the receiver's future sequence values is not empty, removes its first object."

	self nextPutAll: (Array with: anObject).
	^ anObject! !

!NSCollectionStream methodsFor: 'writing' stamp: 'dc 9/20/2007 16:54'!
nextPutAll: aCollection
	(self hasRoomFor: aCollection size)
		ifFalse: [self makeRoomFor: aCollection size].
	self writeAllElementsOf: aCollection.! !

!NSCollectionStream methodsFor: 'accessing' stamp: 'dc 3/10/2008 15:39'!
originalContents
	"Return a collection containing the complete contents of the stream without copying it."
	^ collection! !

!NSCollectionStream methodsFor: 'private - accessing' stamp: 'dc 3/10/2008 15:39'!
outputCollectionClass
	^ collection species! !

!NSCollectionStream methodsFor: 'reading' stamp: 'dc 3/10/2008 15:39'!
peek
	"Returns the next object in the receiver's future sequence values without advancing the receiver's position. Returns nil if the receiver is at end of stream."
	^ self atEnd
		ifFalse: [ collection at: position + 1]
		ifTrue: [nil]! !

!NSCollectionStream methodsFor: 'positioning' stamp: 'dc 9/20/2007 16:28'!
position
	^ position ! !

!NSCollectionStream methodsFor: 'actions' stamp: 'dc 9/27/2007 17:00'!
resetContents
	self reset.
	streamSize := 0.! !

!NSCollectionStream methodsFor: 'private - accessing' stamp: 'dc 9/20/2007 16:28'!
setCollection: aCollection
	collection := aCollection! !

!NSCollectionStream methodsFor: 'private - accessing' stamp: 'dc 9/20/2007 16:28'!
setPosition: anInteger
	position := anInteger ! !

!NSCollectionStream methodsFor: 'accessing' stamp: 'dc 9/20/2007 17:03'!
size
	^ streamSize ! !

!NSCollectionStream methodsFor: 'private - accessing' stamp: 'dc 9/20/2007 17:01'!
streamSize: anInteger
	streamSize := anInteger ! !

!NSCollectionStream methodsFor: 'private' stamp: 'dc 9/20/2007 16:53'!
writeAllElementsOf: aCollection
	collection writeAllElementsOf: aCollection startingAt: position + 1.
	position := position + aCollection size.
	streamSize := streamSize max: position! !

NSCollectionStream subclass: #NSStringStream
	uses: NSTCharacterWriting - {#nextPut:} + NSTCharacterReading
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Classes'!

!NSStringStream commentStamp: 'dc 10/6/2007 15:12' prior: 0!
Same as NSCollectionStream, but with optimizations for strings!

NSStringStream class
	uses: NSTCharacterWriting classTrait + NSTCharacterReading classTrait
	instanceVariableNames: ''!

!NSStringStream methodsFor: 'as yet unclassified' stamp: 'dc 3/10/2008 17:26'!
atEnd
	"Answers true if and only if there is no more element to read."
	<primitive: 67>
	^ position = streamSize! !

!NSStringStream methodsFor: 'initialization' stamp: 'dc 3/10/2008 16:20'!
initialize
	super initialize.
	capacity := collection size.! !

!NSStringStream methodsFor: 'reading' stamp: 'dc 3/10/2008 16:10'!
next
	"Reads the next object in the stream and returns it."
	<primitive: 65>
	^ (position = streamSize)
		ifFalse: [collection at:  (position :=  position + 1)]! !

!NSStringStream methodsFor: 'reading' stamp: 'dc 3/10/2008 15:54'!
next: amount 
	| oldPosition realAmount |
	position := (streamSize min: (oldPosition := position) + amount).
	realAmount := position - oldPosition.
	^ (ByteString new: realAmount) 
		replaceFrom: 1
		to: realAmount
		with: collection
		startingAt: oldPosition + 1! !

!NSStringStream methodsFor: 'writing' stamp: 'dc 3/10/2008 16:19'!
nextPut: anObject 
	| endPosition |
	
	capacity >= (endPosition := position + 1) 
		ifFalse: 
			[collection := (ByteString basicNew: (capacity := 11 + (capacity // 4 max: 20) + position))
						replaceFrom: 1 to: position with: collection  startingAt: 1.
			collection at: endPosition put: anObject.
			streamSize := position := endPosition.
			^ anObject]
		ifTrue: 
			[collection  at: endPosition put: anObject.
			streamSize < (position := endPosition) ifTrue: [streamSize := position].
			^ anObject].! !

!NSStringStream methodsFor: 'writing' stamp: 'damiencassou 2/17/2009 18:20'!
nextPutAll: aCollection 
	| endPosition |
	capacity >= (endPosition := position + aCollection size)
		ifFalse: 
			[collection := (ByteString basicNew: (capacity := 10 + (capacity // 4 max: 20) + endPosition))
								replaceFrom: 1 to: position with: collection startingAt: 1.
								aCollection insertInto: collection from: position + 1 to: endPosition.
			streamSize := position := endPosition.
			^aCollection]
		ifTrue: 
			[aCollection	insertInto: collection from: position + 1 to: endPosition.
			streamSize < (position  := endPosition) ifTrue: [streamSize := position].
			^aCollection]! !

!NSStringStream methodsFor: 'private - accessing' stamp: 'dc 3/10/2008 15:53'!
outputCollectionClass
	^ ByteString ! !

Object subclass: #NSDecoder
	uses: NSTDecoder
	instanceVariableNames: 'atEnd nextValue inputStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Classes'!

NSDecoder class
	uses: NSTDecoder classTrait
	instanceVariableNames: ''!

!NSDecoder methodsFor: 'accessing' stamp: 'dc 5/10/2007 09:25'!
atEnd
	^ atEnd! !

!NSDecoder methodsFor: 'private - accessing' stamp: 'dc 5/10/2007 09:36'!
atEnd: aTemporary 
	atEnd := aTemporary! !

!NSDecoder methodsFor: 'private - accessing' stamp: 'cyrille.delaunay 6/12/2009 10:06'!
inputStream
	^ inputStream ! !

!NSDecoder methodsFor: 'private - accessing' stamp: 'cyrille.delaunay 6/12/2009 10:06'!
inputStream: aGettableStream 
	inputStream  := aGettableStream! !

!NSDecoder methodsFor: 'private - accessing' stamp: 'dc 5/10/2007 09:41'!
nextValue
	^ nextValue.! !

!NSDecoder methodsFor: 'private - accessing' stamp: 'dc 5/10/2007 09:40'!
nextValue: anObject
	nextValue := anObject.! !

!NSDecoder methodsFor: 'private - accessing' stamp: 'cyrille.delaunay 6/12/2009 10:05'!
outputCollectionClass
	^ inputStream outputCollectionClass! !

NSDecoder subclass: #NSPluggableDecoder
	instanceVariableNames: 'decoderBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Nile-Base-Classes'!

!NSPluggableDecoder classSide methodsFor: 'instance creation' stamp: 'dc 5/10/2007 09:30'!
decoderBlock: aOneArgumentBlock inputStream: aStream
	^ self new
		decoderBlock: aOneArgumentBlock;
		inputStream: aStream;
		initializeDecoder;
		yourself! !

!NSPluggableDecoder methodsFor: 'private - accessing' stamp: 'dc 5/5/2007 13:10'!
decoderBlock
	^ decoderBlock! !

!NSPluggableDecoder methodsFor: 'private - accessing' stamp: 'dc 5/5/2007 13:10'!
decoderBlock: anObject
	decoderBlock := anObject! !

!NSPluggableDecoder methodsFor: 'private - accessing' stamp: 'dc 5/5/2007 15:29'!
effectiveNext
	"Reads the input stream and returns a computed value based on it. For private use only."
	"This method has to use #inputStream to access the original stream. This method MUST throw NSStreamAtEndError if it can't read anything interesting in the input stream."
	^ decoderBlock value: self inputStream ! !
